@@ -3,7 +3,7 @@
 import re
 import datetime
 from functools import reduce
-from itertools import islice, izip, chain
+from itertools import islice, izip, chain, ifilter
 
 from sql import Table, Column, Literal, Desc, Asc, Expression, Flavor
 from sql.functions import Now, Extract
@@ -14,7 +14,7 @@ from sql.aggregate import Count, Max
 from trytond.model import ModelStorage, ModelView
 from trytond.model import fields
 from trytond import backend
-from trytond.tools import reduce_ids
+from trytond.tools import reduce_ids, grouped_slice
 from trytond.const import OPERATORS, RECORD_CACHE_SIZE
 from trytond.transaction import Transaction
 from trytond.pool import Pool
@@ -139,8 +139,7 @@ class ModelSQL(ModelStorage):
             if isinstance(field, fields.Many2One) \
                     and field.model_name == cls.__name__ \
                     and field.left and field.right:
-                with Transaction().set_user(0):
-                    cls._rebuild_tree(field_name, None, 0)
+                cls._rebuild_tree(field_name, None, 0)
 
         for ident, constraint, _ in cls._sql_constraints:
             table.add_constraint(ident, constraint)
@@ -189,10 +188,6 @@ class ModelSQL(ModelStorage):
 
     @staticmethod
     def table_query():
-        '''
-        Return None if the model is a real table in the database
-        or return a tuple with the SQL query and the arguments.
-        '''
         return None
 
     @classmethod
@@ -246,9 +241,7 @@ class ModelSQL(ModelStorage):
         table = cls.__table_history__()
         user = User.__table__()
         revisions = []
-        in_max = cursor.IN_MAX
-        for i in range(0, len(ids), in_max):
-            sub_ids = ids[i:i + in_max]
+        for sub_ids in grouped_slice(ids):
             where = reduce_ids(table.id, sub_ids)
             cursor.execute(*table.join(user, 'LEFT',
                     Coalesce(table.write_uid, table.create_uid) == user.id)
@@ -272,7 +265,6 @@ class ModelSQL(ModelStorage):
     def __insert_history(cls, ids, deleted=False):
         transaction = Transaction()
         cursor = transaction.cursor
-        in_max = cursor.IN_MAX
         if not cls._history:
             return
         user = transaction.user
@@ -293,8 +285,7 @@ class ModelSQL(ModelStorage):
                 continue
             columns.append(Column(table, fname))
             hcolumns.append(Column(history, fname))
-        for i in range(0, len(ids), in_max):
-            sub_ids = ids[i:i + in_max]
+        for sub_ids in grouped_slice(ids):
             if not deleted:
                 where = reduce_ids(table.id, sub_ids)
                 cursor.execute(*history.insert(hcolumns,
@@ -310,14 +301,13 @@ class ModelSQL(ModelStorage):
             return
         transaction = Transaction()
         cursor = transaction.cursor
-        in_max = cursor.IN_MAX
         table = cls.__table__()
         history = cls.__table_history__()
         columns = []
         hcolumns = []
-        for fname, field in sorted(cls._fields.iteritems()):
-            if hasattr(field, 'set'):
-                continue
+        fnames = sorted(n for n, f in cls._fields.iteritems()
+            if not hasattr(f, 'set'))
+        for fname in fnames:
             columns.append(Column(table, fname))
             if fname == 'write_uid':
                 hcolumns.append(Literal(transaction.user))
@@ -326,16 +316,20 @@ class ModelSQL(ModelStorage):
             else:
                 hcolumns.append(Column(history, fname))
 
+        def is_deleted(values):
+            return all(not v for n, v in zip(fnames, values)
+                if n not in ['id', 'write_uid', 'write_date'])
+
         to_delete = []
         to_update = []
         for id_ in ids:
             column_datetime = Coalesce(history.write_date, history.create_date)
             hwhere = (column_datetime <= datetime) & (history.id == id_)
-            horder = column_datetime.desc
+            horder = (column_datetime.desc, Column(history, '__id').desc)
             cursor.execute(*history.select(*hcolumns,
                     where=hwhere, order_by=horder, limit=1))
             values = cursor.fetchone()
-            if not values:
+            if not values or is_deleted(values):
                 to_delete.append(id_)
             else:
                 to_update.append(id_)
@@ -351,8 +345,7 @@ class ModelSQL(ModelStorage):
                     cursor.execute(*table.insert(columns, [values]))
 
         if to_delete:
-            for i in range(0, len(to_delete), in_max):
-                sub_ids = to_delete[i:i + in_max]
+            for sub_ids in grouped_slice(to_delete):
                 where = reduce_ids(table.id, sub_ids)
                 cursor.execute(*table.delete(where=where))
             cls.__insert_history(to_delete, True)
@@ -363,12 +356,10 @@ class ModelSQL(ModelStorage):
     def __check_timestamp(cls, ids):
         transaction = Transaction()
         cursor = transaction.cursor
-        in_max = cursor.IN_MAX
         table = cls.__table__()
         if not transaction.timestamp:
             return
-        for i in range(0, len(ids), in_max):
-            sub_ids = ids[i:i + in_max]
+        for sub_ids in grouped_slice(ids):
             where = Or()
             for id_ in sub_ids:
                 try:
@@ -394,7 +385,6 @@ class ModelSQL(ModelStorage):
         cursor = transaction.cursor
         pool = Pool()
         Translation = pool.get('ir.translation')
-        in_max = cursor.IN_MAX
 
         super(ModelSQL, cls).create(vlist)
 
@@ -422,8 +412,7 @@ class ModelSQL(ModelStorage):
                     default.append(f)
 
             if default:
-                defaults = cls.default_get(default, with_rec_name=False,
-                    with_on_change=False)
+                defaults = cls.default_get(default, with_rec_name=False)
                 values.update(cls._clean_defaults(defaults))
 
             insert_columns = [table.create_uid, table.create_date]
@@ -455,15 +444,15 @@ class ModelSQL(ModelStorage):
                 new_ids.append(id_new)
             except DatabaseIntegrityError, exception:
                 with Transaction().new_cursor(), \
-                        Transaction().set_user(0):
+                        Transaction().set_context(_check_access=False):
                     cls.__raise_integrity_error(exception, values)
                 raise
 
         domain = pool.get('ir.rule').domain_get(cls.__name__,
                 mode='create')
         if domain:
-            for i in range(0, len(new_ids), in_max):
-                sub_ids = new_ids[i:i + in_max]
+            for sub_ids in grouped_slice(new_ids):
+                sub_ids = list(sub_ids)
                 red_sql = reduce_ids(table.id, sub_ids)
 
                 cursor.execute(*table.select(table.id,
@@ -499,8 +488,8 @@ class ModelSQL(ModelStorage):
         cls.__insert_history(new_ids)
 
         records = cls.browse(new_ids)
-        for i in range(0, len(records), RECORD_CACHE_SIZE):
-            cls._validate(records[i:i + RECORD_CACHE_SIZE])
+        for sub_records in grouped_slice(records, RECORD_CACHE_SIZE):
+            cls._validate(sub_records)
 
         field_names = cls._fields.keys()
         cls._update_mptt(field_names, [new_ids] * len(field_names))
@@ -575,8 +564,8 @@ class ModelSQL(ModelStorage):
             if 'id' not in fields_names:
                 columns.append(table.id.as_('id'))
 
-            for i in range(0, len(ids), in_max):
-                sub_ids = ids[i:i + in_max]
+            for sub_ids in grouped_slice(ids, in_max):
+                sub_ids = list(sub_ids)
                 red_sql = reduce_ids(table.id, sub_ids)
                 where = red_sql
                 if history_clause:
@@ -745,7 +734,6 @@ class ModelSQL(ModelStorage):
         pool = Pool()
         Translation = pool.get('ir.translation')
         Config = pool.get('ir.configuration')
-        in_max = cursor.IN_MAX
 
         assert not len(args) % 2
         all_records = sum(((records, values) + args)[0:None:2], [])
@@ -787,8 +775,8 @@ class ModelSQL(ModelStorage):
                         update_values.append(field.sql_format(value))
 
             domain = pool.get('ir.rule').domain_get(cls.__name__, mode='write')
-            for i in range(0, len(ids), in_max):
-                sub_ids = ids[i:i + in_max]
+            for sub_ids in grouped_slice(ids):
+                sub_ids = list(sub_ids)
                 red_sql = reduce_ids(table.id, sub_ids)
                 where = red_sql
                 if domain:
@@ -811,7 +799,7 @@ class ModelSQL(ModelStorage):
                             where=red_sql))
                 except DatabaseIntegrityError, exception:
                     with Transaction().new_cursor(), \
-                            Transaction().set_user(0):
+                            Transaction().set_context(_check_access=False):
                         cls.__raise_integrity_error(exception, values,
                             values.keys())
                     raise
@@ -826,18 +814,17 @@ class ModelSQL(ModelStorage):
                 if hasattr(field, 'set'):
                     fields_to_set.setdefault(fname, []).extend((ids, value))
 
-            field_names = cls._fields.keys()
+            field_names = values.keys()
             cls._update_mptt(field_names, [ids] * len(field_names), values)
-            all_field_names |= set(values.keys())
+            all_field_names |= set(field_names)
 
         for fname, fargs in fields_to_set.iteritems():
             field = cls._fields[fname]
             field.set(cls, fname, *fargs)
 
         cls.__insert_history(all_ids)
-        for i in range(0, len(all_records), RECORD_CACHE_SIZE):
-            cls._validate(all_records[i:i + RECORD_CACHE_SIZE],
-                field_names=all_field_names)
+        for sub_records in grouped_slice(all_records, RECORD_CACHE_SIZE):
+            cls._validate(sub_records, field_names=all_field_names)
         cls.trigger_write(trigger_eligibles)
 
     @classmethod
@@ -848,7 +835,6 @@ class ModelSQL(ModelStorage):
         pool = Pool()
         Translation = pool.get('ir.translation')
         ids = map(int, records)
-        in_max = cursor.IN_MAX
 
         if not ids:
             return
@@ -871,8 +857,7 @@ class ModelSQL(ModelStorage):
                     and field.model_name == cls.__name__
                     and field.left and field.right):
                 tree_ids[fname] = []
-                for i in range(0, len(ids), in_max):
-                    sub_ids = ids[i:i + in_max]
+                for sub_ids in grouped_slice(ids):
                     where = reduce_ids(Column(table, fname), sub_ids)
                     cursor.execute(*table.select(table.id, where=where))
                     tree_ids[fname] += [x[0] for x in cursor.fetchall()]
@@ -903,8 +888,8 @@ class ModelSQL(ModelStorage):
         domain = pool.get('ir.rule').domain_get(cls.__name__, mode='delete')
 
         if domain:
-            for i in range(0, len(ids), in_max):
-                sub_ids = ids[i:i + in_max]
+            for sub_ids in grouped_slice(ids):
+                sub_ids = list(sub_ids)
                 red_sql = reduce_ids(table.id, sub_ids)
                 cursor.execute(*table.select(table.id,
                         where=red_sql & table.id.in_(domain)))
@@ -916,9 +901,9 @@ class ModelSQL(ModelStorage):
 
         cls.trigger_delete(records)
 
-        for i in range(0, len(ids), in_max):
-            sub_ids = ids[i:i + in_max]
-            sub_records = records[i:i + in_max]
+        for sub_ids, sub_records in izip(
+                grouped_slice(ids), grouped_slice(records)):
+            sub_ids = list(sub_ids)
             red_sql = reduce_ids(table.id, sub_ids)
 
             transaction.delete_records.setdefault(cls.__name__,
@@ -953,7 +938,7 @@ class ModelSQL(ModelStorage):
                     Model.delete(models)
 
             for Model, field_name in foreign_keys_tocheck:
-                with Transaction().set_user(0):
+                with Transaction().set_context(_check_access=False):
                     if Model.search([
                                 (field_name, 'in', sub_ids),
                                 ], order=[]):
@@ -961,7 +946,7 @@ class ModelSQL(ModelStorage):
                         cls.raise_user_error('foreign_model_exist',
                             error_args=error_args)
 
-            super(ModelSQL, cls).delete(sub_records)
+            super(ModelSQL, cls).delete(list(sub_records))
 
             try:
                 cursor.execute(*table.delete(where=red_sql))
@@ -983,7 +968,6 @@ class ModelSQL(ModelStorage):
         Rule = pool.get('ir.rule')
         transaction = Transaction()
         cursor = transaction.cursor
-        in_max = cursor.IN_MAX
 
         # Get domain clauses
         tables, expression = cls.search_domain(domain)
@@ -1034,6 +1018,7 @@ class ModelSQL(ModelStorage):
             columns.append(Coalesce(
                     main_table.write_date,
                     main_table.create_date).as_('_datetime'))
+            columns.append(Column(main_table, '__id'))
         if not query:
             columns += [Column(main_table, name).as_(name)
                 for name, field in cls._fields.iteritems()
@@ -1053,11 +1038,50 @@ class ModelSQL(ModelStorage):
         cursor.execute(*select)
 
         rows = cursor.dictfetchmany(cursor.IN_MAX)
-        cache = cursor.get_cache(transaction.context)
+        cache = cursor.get_cache()
         if cls.__name__ not in cache:
             cache[cls.__name__] = LRUDict(RECORD_CACHE_SIZE)
         delete_records = transaction.delete_records.setdefault(cls.__name__,
             set())
+
+        def filter_history(rows):
+            if not (cls._history and transaction.context.get('_datetime')):
+                return rows
+
+            def history_key(row):
+                return row['_datetime'], row['__id']
+
+            ids_history = {}
+            for row in rows:
+                key = history_key(row)
+                if row['id'] in ids_history:
+                    if key < ids_history[row['id']]:
+                        continue
+                ids_history[row['id']] = key
+
+            to_delete = set()
+            history = cls.__table_history__()
+            for sub_ids in grouped_slice([r['id'] for r in rows]):
+                where = reduce_ids(history.id, sub_ids)
+                cursor.execute(*history.select(history.id, history.write_date,
+                        where=where
+                        & (history.write_date != None)
+                        & (history.create_date == None)
+                        & (history.write_date
+                            <= transaction.context['_datetime'])))
+                for deleted_id, delete_date in cursor.fetchall():
+                    history_date, _ = ids_history[deleted_id]
+                    if isinstance(history_date, basestring):
+                        strptime = datetime.datetime.strptime
+                        format_ = '%Y-%m-%d %H:%M:%S.%f'
+                        history_date = strptime(history_date, format_)
+                    if history_date <= delete_date:
+                        to_delete.add(deleted_id)
+
+            return ifilter(lambda r: history_key(r) == ids_history[r['id']]
+                and r['id'] not in to_delete, rows)
+
+        rows = list(filter_history(rows))
         keys = None
         for data in islice(rows, 0, cache.size_limit):
             if data['id'] in delete_records:
@@ -1065,7 +1089,7 @@ class ModelSQL(ModelStorage):
             if keys is None:
                 keys = data.keys()
                 for k in keys[:]:
-                    if k in ('_timestamp', '_datetime'):
+                    if k in ('_timestamp', '_datetime', '__id'):
                         keys.remove(k)
                         continue
                     field = cls._fields[k]
@@ -1086,34 +1110,7 @@ class ModelSQL(ModelStorage):
             cursor.execute(*table.select(*columns,
                     where=expression, order_by=order_by,
                     limit=limit, offset=offset))
-            rows = cursor.dictfetchall()
-
-        if cls._history and transaction.context.get('_datetime'):
-            ids = []
-            ids_date = {}
-            for data in rows:
-                if data['id'] in ids_date:
-                    if data['_datetime'] <= ids_date[data['id']]:
-                        continue
-                if data['id'] in ids:
-                    ids.remove(data['id'])
-                ids.append(data['id'])
-                ids_date[data['id']] = data['_datetime']
-            to_delete = set()
-            history = cls.__table_history__()
-            for i in range(0, len(ids), in_max):
-                sub_ids = ids[i:i + in_max]
-                where = reduce_ids(history.id, sub_ids)
-                cursor.execute(*history.select(history.id, history.write_date,
-                        where=where
-                        & (history.write_date != None)
-                        & (history.create_date == None)
-                        & (history.write_date
-                            <= transaction.context['_datetime'])))
-                for deleted_id, delete_date in cursor.fetchall():
-                    if ids_date[deleted_id] < delete_date:
-                        to_delete.add(deleted_id)
-            return cls.browse(filter(lambda x: x not in to_delete, ids))
+            rows = filter_history(cursor.dictfetchall())
 
         return cls.browse([x['id'] for x in rows])
 
@@ -1184,8 +1181,7 @@ class ModelSQL(ModelStorage):
                         cls._update_tree(id_, field_name,
                             field.left, field.right)
                 else:
-                    with Transaction().set_user(0):
-                        cls._rebuild_tree(field_name, None, 0)
+                    cls._rebuild_tree(field_name, None, 0)
 
     @classmethod
     def _rebuild_tree(cls, parent, parent_id, left):
@@ -1291,8 +1287,7 @@ class ModelSQL(ModelStorage):
                 sql_clause = '(id != ' + param + ' AND ' + sql_clause + ')'
 
                 in_max = cursor.IN_MAX / (len(columns) + 1)
-                for i in range(0, len(ids), in_max):
-                    sub_ids = ids[i:i + in_max]
+                for sub_ids in grouped_slice(ids, in_max):
                     red_sql = reduce_ids(table.id, sub_ids)
 
                     cursor.execute('SELECT id,' + sql + ' '
@@ -1312,8 +1307,7 @@ class ModelSQL(ModelStorage):
             match = _RE_CHECK.match(sql)
             if match:
                 sql = match.group(1)
-                for i in range(0, len(ids), cursor.IN_MAX):
-                    sub_ids = ids[i:i + cursor.IN_MAX]
+                for sub_ids in grouped_slice(ids):
                     red_sql = reduce_ids(table.id, sub_ids)
                     cursor.execute('SELECT id '
                         'FROM "' + cls._table + '" '
