@@ -253,7 +253,7 @@ class ModelView(Model):
             - relate: a list of available relations
         """
         Action = Pool().get('ir.action.keyword')
-        key = (cls.__name__, repr(Transaction().context))
+        key = cls.__name__
         result = cls._view_toolbar_get_cache.get(key)
         if result:
             return result
@@ -426,7 +426,9 @@ class ModelView(Model):
         # convert attributes into pyson
         encoder = PYSONEncoder()
         for attr in ('states', 'domain', 'spell', 'colors'):
-            if element.get(attr):
+            if (element.get(attr)
+                    # Avoid double evaluation from inherit with different model
+                    and '__' not in element.get(attr)):
                 element.set(attr, encoder.encode(safe_eval(element.get(attr),
                     CONTEXT)))
 
@@ -476,17 +478,22 @@ class ModelView(Model):
         @wraps(func)
         def wrapper(cls, *args, **kwargs):
             pool = Pool()
+            ModelAccess = pool.get('ir.model.access')
             Button = pool.get('ir.model.button')
             User = pool.get('res.user')
 
-            if Transaction().user != 0:
+            if ((Transaction().user != 0)
+                    and Transaction().context.get('_check_access')):
+                ModelAccess.check(cls.__name__, 'read')
+                ModelAccess.check(cls.__name__, 'write')
                 groups = set(User.get_groups())
                 button_groups = Button.get_groups(cls.__name__,
                     func.__name__)
                 if button_groups and not groups & button_groups:
                     raise UserError('Calling button %s on %s is not allowed!'
                         % (func.__name__, cls.__name__))
-            return func(cls, *args, **kwargs)
+            with Transaction().set_context(_check_access=False):
+                return func(cls, *args, **kwargs)
         return wrapper
 
     @staticmethod
