@@ -2,7 +2,6 @@
 #this repository contains the full copyright notices and license terms.
 from trytond.protocols.sslsocket import SSLSocket
 from trytond.protocols.dispatcher import dispatch
-from trytond.config import CONFIG
 from trytond.protocols.common import daemon, RegisterHandlerMixin
 from trytond.exceptions import UserError, UserWarning, NotLogged, \
     ConcurrencyException
@@ -10,16 +9,18 @@ from trytond import security
 import SimpleXMLRPCServer
 import SocketServer
 import xmlrpclib
-import traceback
 import socket
 import sys
 import base64
 import datetime
 from types import DictType
+import logging
 
 # convert decimal to float before marshalling:
 from decimal import Decimal
 
+logger = logging.getLogger(__name__)
+
 
 def dump_decimal(self, value, write):
     value = {'__class__': 'Decimal',
@@ -55,7 +56,7 @@ def dump_time(self, value, write):
 
 xmlrpclib.Marshaller.dispatch[Decimal] = dump_decimal
 xmlrpclib.Marshaller.dispatch[type(None)] = \
-        lambda self, value, write: self.dump_bool(bool(value), write)
+        lambda self, value, write: write("<value><nil/></value>")
 xmlrpclib.Marshaller.dispatch[datetime.date] = dump_date
 xmlrpclib.Marshaller.dispatch[datetime.time] = dump_time
 xmlrpclib.Marshaller.dispatch[buffer] = dump_buffer
@@ -74,6 +75,28 @@ def dump_struct(self, value, write, escape=xmlrpclib.escape):
 xmlrpclib.Marshaller.dispatch[DictType] = dump_struct
 
 
+class XMLRPCDecoder(object):
+
+    decoders = {}
+
+    @classmethod
+    def register(cls, klass, decoder):
+        assert klass not in cls.decoders
+        cls.decoders[klass] = decoder
+
+    def __call__(self, dct):
+        if dct.get('__class__') in self.decoders:
+            return self.decoders[dct['__class__']](dct)
+        return dct
+
+XMLRPCDecoder.register('date',
+    lambda dct: datetime.date(dct['year'], dct['month'], dct['day']))
+XMLRPCDecoder.register('time',
+    lambda dct: datetime.time(dct['hour'], dct['minute'], dct['second'],
+        dct['microsecond']))
+XMLRPCDecoder.register('Decimal', lambda dct: Decimal(dct['decimal']))
+
+
 def end_struct(self, data):
     mark = self._marks.pop()
     # map structs to Python dictionaries
@@ -81,14 +104,7 @@ def end_struct(self, data):
     items = self._stack[mark:]
     for i in range(0, len(items), 2):
         dct[xmlrpclib._stringify(items[i])] = items[i + 1]
-    if '__class__' in dct:
-        if dct['__class__'] == 'date':
-            dct = datetime.date(dct['year'], dct['month'], dct['day'])
-        elif dct['__class__'] == 'time':
-            dct = datetime.time(dct['hour'], dct['minute'], dct['second'],
-                dct['microsecond'])
-        elif dct['__class__'] == 'Decimal':
-            dct = Decimal(dct['decimal'])
+    dct = XMLRPCDecoder()(dct)
     self._stack[mark:] = [dct]
     self._value = 0
 
@@ -103,6 +119,14 @@ def _end_dateTime(self, data):
 xmlrpclib.Unmarshaller.dispatch["dateTime.iso8601"] = _end_dateTime
 
 
+def _end_base64(self, data):
+    value = xmlrpclib.Binary()
+    value.decode(data)
+    self.append(buffer(value.data))
+    self._value = 0
+xmlrpclib.Unmarshaller.dispatch['base64'] = _end_base64
+
+
 class GenericXMLRPCRequestHandler:
 
     def _dispatch(self, method, params):
@@ -110,6 +134,7 @@ class GenericXMLRPCRequestHandler:
         database_name = self.path[1:]
         user = self.tryton['user']
         session = self.tryton['session']
+        exception_message = 'Exception calling %s%s' % (method, params)
         try:
             try:
                 method_list = method.split('.')
@@ -127,21 +152,15 @@ class GenericXMLRPCRequestHandler:
                 return dispatch(host, port, 'XML-RPC', database_name, user,
                         session, object_type, object_name, method, *params)
             except (NotLogged, ConcurrencyException), exception:
-                raise xmlrpclib.Fault(exception.code,
-                    '\n'.join(exception.args))
+                logger.debug(exception_message, exc_info=sys.exc_info())
+                raise xmlrpclib.Fault(exception.code, str(exception))
             except (UserError, UserWarning), exception:
+                logger.debug(exception_message, exc_info=sys.exc_info())
                 error, description = exception.args
-                raise xmlrpclib.Fault(exception.code,
-                    '\n'.join((error,) + description))
-            except Exception:
-                tb_s = ''.join(traceback.format_exception(*sys.exc_info()))
-                for path in sys.path:
-                    tb_s = tb_s.replace(path, '')
-                if CONFIG['debug_mode']:
-                    import pdb
-                    traceb = sys.exc_info()[2]
-                    pdb.post_mortem(traceb)
-                raise xmlrpclib.Fault(255, str(sys.exc_value) + '\n' + tb_s)
+                raise xmlrpclib.Fault(exception.code, str(exception))
+            except Exception, exception:
+                logger.error(exception_message, exc_info=sys.exc_info())
+                raise xmlrpclib.Fault(255, str(exception))
         finally:
             security.logout(database_name, user, session)
 
