@@ -4,18 +4,22 @@
 """
 Miscelleanous tools used by tryton
 """
-import os, sys
+import os
+import sys
 import subprocess
 from threading import local
 import smtplib
-try:
-    import cStringIO as StringIO
-except ImportError:
-    import StringIO
 import dis
 from decimal import Decimal
-from trytond.config import CONFIG
+from array import array
+from itertools import islice
+from sql import Literal
+from sql.operators import Or
+
 from trytond.const import OPERATORS
+from trytond.config import config, parse_uri
+from trytond.transaction import Transaction
+
 
 def find_in_path(name):
     if os.name == "nt":
@@ -30,52 +34,27 @@ def find_in_path(name):
             return val
     return name
 
-def find_pg_tool(name):
-    if CONFIG['pg_path'] and CONFIG['pg_path'] != 'None':
-        return os.path.join(CONFIG['pg_path'], name)
-    else:
-        return find_in_path(name)
 
-def exec_pg_command(name, *args):
-    prog = find_pg_tool(name)
-    if not prog:
-        raise Exception('Couldn\'t find %s' % name)
-    args2 = (os.path.basename(prog),) + args
-    return os.spawnv(os.P_WAIT, prog, args2)
-
-def exec_pg_command_pipe(name, *args):
-    prog = find_pg_tool(name)
+def exec_command_pipe(name, *args, **kwargs):
+    prog = find_in_path(name)
     if not prog:
         raise Exception('Couldn\'t find %s' % name)
     if os.name == "nt":
         cmd = '"' + prog + '" ' + ' '.join(args)
     else:
         cmd = prog + ' ' + ' '.join(args)
-
-    # if db_password is set in configuration we should pass
-    # an environment variable PGPASSWORD to our subprocess
-    # see libpg documentation
     child_env = dict(os.environ)
-    if CONFIG['db_password']:
-        child_env['PGPASSWORD'] = CONFIG['db_password']
-    pipe = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE, env=child_env)
-    return pipe
+    if kwargs.get('env'):
+        child_env.update(kwargs['env'])
+    return subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
+        stdout=subprocess.PIPE, env=child_env)
 
-def exec_command_pipe(name, *args):
-    prog = find_in_path(name)
-    if not prog:
-        raise Exception('Couldn\'t find %s' % name)
-    if os.name == "nt":
-        cmd = '"'+prog+'" '+' '.join(args)
-    else:
-        cmd = prog+' '+' '.join(args)
-    return os.popen2(cmd, 'b')
 
 def file_open(name, mode="r", subdir='modules'):
     """Open a file from the root dir, using a subdir folder."""
     from trytond.modules import EGG_MODULES
-    root_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
+    root_path = os.path.dirname(os.path.dirname(os.path.abspath(
+                unicode(__file__, sys.getfilesystemencoding()))))
 
     egg_name = False
     if subdir == 'modules':
@@ -100,12 +79,11 @@ def file_open(name, mode="r", subdir='modules'):
                         os.path.dirname(epoint.dist.location), name)
 
     if subdir:
-        if subdir == 'modules'\
-                and (name.startswith('ir' + os.sep) \
-                    or name.startswith('workflow' + os.sep) \
-                    or name.startswith('res' + os.sep) \
-                    or name.startswith('webdav' + os.sep) \
-                    or name.startswith('test' + os.sep)):
+        if (subdir == 'modules'
+                and (name.startswith('ir' + os.sep)
+                    or name.startswith('res' + os.sep)
+                    or name.startswith('webdav' + os.sep)
+                    or name.startswith('tests' + os.sep))):
             name = os.path.join(root_path, name)
         else:
             name = os.path.join(root_path, subdir, name)
@@ -118,6 +96,7 @@ def file_open(name, mode="r", subdir='modules'):
 
     raise IOError('File not found : %s ' % name)
 
+
 def get_smtp_server():
     """
     Instanciate, configure and return a SMTP or SMTP_SSL instance from
@@ -125,28 +104,20 @@ def get_smtp_server():
     :return: A SMTP instance. The quit() method must be call when all
     the calls to sendmail() have been made.
     """
-    if CONFIG['smtp_ssl']:
-        smtp_server = smtplib.SMTP_SSL(CONFIG['smtp_server'],
-                CONFIG['smtp_port'])
+    uri = parse_uri(config.get('email', 'uri'))
+    if uri.scheme.startswith('smtps'):
+        smtp_server = smtplib.SMTP_SSL(uri.hostname, uri.port)
     else:
-        smtp_server = smtplib.SMTP(CONFIG['smtp_server'], CONFIG['smtp_port'])
+        smtp_server = smtplib.SMTP(uri.hostname, uri.port)
 
-    if CONFIG['smtp_tls']:
+    if 'tls' in uri.scheme:
         smtp_server.starttls()
 
-    if CONFIG['smtp_user'] and CONFIG['smtp_password']:
-        smtp_server.login(CONFIG['smtp_user'], CONFIG['smtp_password'])
+    if uri.username and uri.password:
+        smtp_server.login(uri.username, uri.password)
 
     return smtp_server
 
-def find_language_context(args, kargs=None):
-    if kargs is None:
-        kargs = {}
-    res = 'en_US'
-    for arg in args:
-        if isinstance(arg, dict):
-            res = arg.get('language', 'en_US')
-    return kargs.get('context', {}).get('language', res)
 
 def memoize(maxsize):
     """
@@ -210,6 +181,7 @@ def memoize(maxsize):
         return wrapper
     return wrap
 
+
 def mod10r(number):
     """
     Recursive mod10
@@ -223,7 +195,7 @@ def mod10r(number):
     for digit in number:
         result += digit
         if digit.isdigit():
-            report = codec[ (int(digit) + report) % 10 ]
+            report = codec[(int(digit) + report) % 10]
     return result + str((10 - report) % 10)
 
 
@@ -282,7 +254,7 @@ class LocalDict(local):
         return self._dict.get(k, d)
 
     def has_key(self, k):
-        return self._dict.has_key(k)
+        return k in self._dict
 
     def items(self):
         return self._dict.items()
@@ -302,9 +274,6 @@ class LocalDict(local):
     def __ge__(self, y):
         return self._dict.__ge__(y)
 
-    def __getitem__(self, y):
-        return self._dict.__getitem__(y)
-
     def __gt__(self, y):
         return self._dict.__gt__(y)
 
@@ -326,23 +295,22 @@ class LocalDict(local):
     def __ne__(self, y):
         return self._dict.__ne__(y)
 
+
 def reduce_ids(field, ids):
     '''
-    Return a small SQL clause for ids
-
-    :param field: the field of the clause
-    :param ids: the list of ids
-    :return: sql string and sql param
+    Return a small SQL expression for the list of ids and the sql column
     '''
+    ids = list(ids)
     if not ids:
-        return '(%s)', [False]
-    ids = ids[:]
+        return Literal(False)
+    assert all(x.is_integer() for x in ids if isinstance(x, float)), \
+        'ids must be integer'
+    ids = map(int, ids)
     ids.sort()
     prev = ids.pop(0)
     continue_list = [prev, prev]
-    discontinue_list = []
-    sql = []
-    args = []
+    discontinue_list = array('l')
+    sql = Or()
     for i in ids:
         if i == prev:
             continue
@@ -351,40 +319,35 @@ def reduce_ids(field, ids):
                 discontinue_list.extend([continue_list[0] + x for x in
                     range(continue_list[-1] - continue_list[0] + 1)])
             else:
-                sql.append('((' + field + ' >= %s) AND (' + field + ' <= %s))')
-                args.append(continue_list[0])
-                args.append(continue_list[-1])
+                sql.append((field >= continue_list[0])
+                    & (field <= continue_list[-1]))
             continue_list = []
         continue_list.append(i)
         prev = i
     if continue_list[-1] - continue_list[0] < 5:
         discontinue_list.extend([continue_list[0] + x for x in
-            range(int(continue_list[-1] - continue_list[0] + 1))])
+            range(continue_list[-1] - continue_list[0] + 1)])
     else:
-        sql.append('((' + field + ' >= %s) AND (' + field + ' <= %s))')
-        args.append(continue_list[0])
-        args.append(continue_list[-1])
+        sql.append((field >= continue_list[0]) & (field <= continue_list[-1]))
     if discontinue_list:
-        sql.append('(' + field + ' IN (' + \
-                ','.join(('%s',) * len(discontinue_list)) + '))')
-        args.extend(discontinue_list)
-    return '(' + ' OR '.join(sql) + ')', args
+        sql.append(field.in_(discontinue_list))
+    return sql
 
 _ALLOWED_CODES = set(dis.opmap[x] for x in [
-    'POP_TOP','ROT_TWO','ROT_THREE','ROT_FOUR','DUP_TOP',
-    'BUILD_LIST','BUILD_MAP','BUILD_TUPLE',
-    'LOAD_CONST','RETURN_VALUE','STORE_SUBSCR',
-    'UNARY_POSITIVE','UNARY_NEGATIVE','UNARY_NOT',
-    'UNARY_INVERT','BINARY_POWER','BINARY_MULTIPLY',
-    'BINARY_DIVIDE','BINARY_FLOOR_DIVIDE','BINARY_TRUE_DIVIDE',
-    'BINARY_MODULO','BINARY_ADD','BINARY_SUBTRACT',
-    'BINARY_LSHIFT','BINARY_RSHIFT','BINARY_AND','BINARY_XOR', 'BINARY_OR',
-    'STORE_MAP', 'LOAD_NAME', 'CALL_FUNCTION', 'COMPARE_OP', 'LOAD_ATTR',
-    'STORE_NAME', 'GET_ITER', 'FOR_ITER', 'LIST_APPEND', 'JUMP_ABSOLUTE',
-    'DELETE_NAME', 'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'JUMP_IF_FALSE_OR_POP',
-    'JUMP_IF_TRUE_OR_POP', 'POP_JUMP_IF_FALSE', 'POP_JUMP_IF_TRUE',
-    'BINARY_SUBSCR',
-    ] if x in dis.opmap)
+        'POP_TOP', 'ROT_TWO', 'ROT_THREE', 'ROT_FOUR', 'DUP_TOP', 'BUILD_LIST',
+        'BUILD_MAP', 'BUILD_TUPLE', 'LOAD_CONST', 'RETURN_VALUE',
+        'STORE_SUBSCR', 'UNARY_POSITIVE', 'UNARY_NEGATIVE', 'UNARY_NOT',
+        'UNARY_INVERT', 'BINARY_POWER', 'BINARY_MULTIPLY', 'BINARY_DIVIDE',
+        'BINARY_FLOOR_DIVIDE', 'BINARY_TRUE_DIVIDE', 'BINARY_MODULO',
+        'BINARY_ADD', 'BINARY_SUBTRACT', 'BINARY_LSHIFT', 'BINARY_RSHIFT',
+        'BINARY_AND', 'BINARY_XOR', 'BINARY_OR', 'STORE_MAP', 'LOAD_NAME',
+        'CALL_FUNCTION', 'COMPARE_OP', 'LOAD_ATTR', 'STORE_NAME', 'GET_ITER',
+        'FOR_ITER', 'LIST_APPEND', 'JUMP_ABSOLUTE', 'DELETE_NAME',
+        'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'JUMP_IF_FALSE_OR_POP',
+        'JUMP_IF_TRUE_OR_POP', 'POP_JUMP_IF_FALSE', 'POP_JUMP_IF_TRUE',
+        'BINARY_SUBSCR', 'JUMP_FORWARD',
+        ] if x in dis.opmap)
+
 
 @memoize(1000)
 def _compile_source(source):
@@ -404,9 +367,10 @@ def _compile_source(source):
             raise ValueError('opcode %s not allowed' % dis.opname[code])
     return comp
 
+
 def safe_eval(source, data=None):
-    if '__subclasses__' in source:
-        raise ValueError('__subclasses__ not allowed')
+    if '__' in source:
+        raise ValueError('Double underscores not allowed')
 
     comp = _compile_source(source)
     return eval(comp, {'__builtins__': {
@@ -421,6 +385,7 @@ def safe_eval(source, data=None):
         'Decimal': Decimal,
         }}, data)
 
+
 def reduce_domain(domain):
     '''
     Reduce domain
@@ -450,3 +415,11 @@ def reduce_domain(domain):
         else:
             result.append(arg)
     return result
+
+
+def grouped_slice(records, count=None):
+    'Grouped slice'
+    if count is None:
+        count = Transaction().cursor.IN_MAX
+    for i in xrange(0, len(records), count):
+        yield islice(records, i, i + count)
